using Microsoft.AspNetCore.Mvc;
using System;
using System.Linq;
using NwndSpAPI.BusinessObject;
using NwndSpAPI.Models;
using NwndSpAPI.ViewModels;
using NwndSpAPI.Domain;
using System.Collections.Generic;
using Microsoft.Extensions.Options;
using System.Threading.Tasks;
using Newtonsoft.Json;
using System.Text;

namespace NwndSp.Controllers.Base
{
     /// <summary>
     /// Base class for CategoriesController.  Do not make changes to this class,
     /// instead, put additional code in the CategoriesController class 
     /// </summary>
     public class CategoriesControllerBase : Controller
     {

         /// <summary>
         /// GET: /Categories/
         /// </summary>
         public IActionResult Index()
         {
             return View();
         }

         /// <summary>
         /// GET: /Categories/Add
         /// </summary>
         public IActionResult Add()
         {
             return GetAddViewModel();
         }

         /// <summary>
         /// POST: /Categories/Add
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public IActionResult Add(CategoriesViewModel viewModel, string returnUrl)
         {
             if (ModelState.IsValid)
             {
                 try
                 {
                     // add new record
                     AddEditCategories(viewModel, CrudOperation.Add);

                     if (Url.IsLocalUrl(returnUrl))
                         return Redirect(returnUrl);
                     else
                         return RedirectToAction("ListCrudRedirect", "Categories");
                 }
                 catch(Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             return GetAddViewModel();
         }

         private IActionResult GetAddViewModel()
         {
             CategoriesViewModel viewModel = new CategoriesViewModel();
             viewModel.CategoriesModel = null;
             viewModel.Operation = CrudOperation.Add;
             viewModel.ViewControllerName = "Categories";
             viewModel.ViewActionName = "Add";
             viewModel.ViewReturnUrl = "/Categories/ListCrudRedirect";

             return View(viewModel);
         }

         /// <summary>
         /// GET: /Categories/Update/5
         /// </summary>
         public IActionResult Update(int id)
         {
             return GetUpdateViewModel(id);
         }

         /// <summary>
         /// POST: /Categories/Update/5
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public IActionResult Update(int id, CategoriesViewModel viewModel, string returnUrl)
         {
             if (ModelState.IsValid)
             {
                 try
                 {
                     // update record
                     AddEditCategories(viewModel, CrudOperation.Update);

                     if (Url.IsLocalUrl(returnUrl))
                         return Redirect(returnUrl);
                     else
                         return RedirectToAction("ListCrudRedirect", "Categories");
                 }
                 catch(Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             return GetUpdateViewModel(id);
         }

         public IActionResult GetUpdateViewModel(int id)
         {
             // select a record by primary key(s)
             Categories objCategories = Categories.SelectByPrimaryKey(id);

             // assign values to the model
             CategoriesModel model = new CategoriesModel();
             model.CategoryID = objCategories.CategoryID;
             model.CategoryName = objCategories.CategoryName;
             model.Description = objCategories.Description;

             // assign values to the view model
             CategoriesViewModel viewModel = new CategoriesViewModel();
             viewModel.CategoriesModel = model;
             viewModel.Operation = CrudOperation.Update;
             viewModel.ViewControllerName = "Categories";
             viewModel.ViewActionName = "Update";

             viewModel.ViewReturnUrl = "/Categories/ListCrudRedirect";
             viewModel.URLReferrer = Request.Headers["Referer"].ToString();

             return View(viewModel);
         }

         /// <summary>
         /// POST: /Categories/Delete/5
         /// </summary>
         [HttpPost]
         public IActionResult Delete(int id, CategoriesViewModel viewModel, string returnUrl)
         {
             Categories.Delete(id);
             return Json(true);
         }

         /// <summary>
         /// GET: /Categories/Details/5
         /// </summary>
         public IActionResult Details(int id)
         {
             // select a record by primary key(s)
             Categories objCategories = Categories.SelectByPrimaryKey(id);

             // assign values to the model
             CategoriesModel model = new CategoriesModel();
             model.CategoryID = objCategories.CategoryID;
             model.CategoryName = objCategories.CategoryName;
             model.Description = objCategories.Description;

             // assign values to the view model
             CategoriesViewModel viewModel = new CategoriesViewModel();
             viewModel.CategoriesModel = model;

             viewModel.ViewReturnUrl = "/Categories/ListCrudRedirect";
             viewModel.URLReferrer = Request.Headers["Referer"].ToString();

             return View(viewModel);
         }

         /// <summary>
         /// GET: /Categories/ListCrudRedirect
         /// </summary>
         public IActionResult ListCrudRedirect()
         {
             return View();
         }

         /// <summary>
         /// GET: /Categories/ListReadOnly
         /// </summary>
         public IActionResult ListReadOnly()
         {
             return View();
         }

         /// <summary>
         /// GET: /Categories/ListCrud
         /// </summary>
         public IActionResult ListCrud()
         {
             return View(GetViewModel("ListCrud"));
         }

         /// <summary>
         /// POST: /Categories/ListCrud
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public IActionResult ListCrud(string inputSubmit, CategoriesViewModel viewModel)
         {
             if (ModelState.IsValid)
             {
                 CrudOperation operation = CrudOperation.Add;

                 if (inputSubmit == "Update")
                     operation = CrudOperation.Update;

                 try
                 {
                     AddEditCategories(viewModel, operation);
                 }
                 catch(Exception ex)
                 {
                     ModelState.AddModelError("", ex.Message);
                 }
             }

             return View(GetViewModel("ListCrud"));
         }

         /// <summary>
         /// GET: /Categories/ListSearch
         /// </summary>
         public IActionResult ListSearch()
         {
             return View(GetViewModel("ListSearch"));
         }

         /// <summary>
         /// GET: /Categories/ListScrollLoad
         /// </summary>
         public IActionResult ListScrollLoad()
         {
             return View();
         }

         /// <summary>
         /// GET: /Categories/ListInline
         /// </summary>
         public IActionResult ListInline()
         {
             return View(GetViewModel("ListInline"));
         }

         /// <summary>
         /// POST: /Categories/ListInlineAdd
         /// </summary>
         [HttpPost]
         public IActionResult ListInlineAdd([FromBody]CategoriesModel model)
         {
             CategoriesViewModel viewModel = new CategoriesViewModel();
             viewModel.CategoriesModel = model;

             AddEditCategories(viewModel, CrudOperation.Add, true);
             return Json("");
         }

         /// <summary>
         /// POST: /Categories/ListInlineUpdate
         /// </summary>
         [HttpPost]
         public IActionResult ListInlineUpdate([FromBody]CategoriesModel model)
         {
             CategoriesViewModel viewModel = new CategoriesViewModel();
             viewModel.CategoriesModel = model;

             AddEditCategories(viewModel, CrudOperation.Update, true);
             return Json("");
         }

         /// <summary>
         /// GET: /Categories/ListForeach
         /// </summary>
         public IActionResult ListForeach(string sidx, string sord, int? page)
         {
             int rows = Functions.GetGridNumberOfRows();
             int numberOfPagesToShow = Functions.GetGridNumberOfPagesToShow();
             int currentPage = page is null ? 1 : Convert.ToInt32(page);
             int startRowIndex = ((currentPage * rows) - rows) + 1;
             int totalRecords = Categories.GetRecordCount();
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);
             List<Categories> objCategoriesCol = Categories.SelectSkipAndTake(rows, startRowIndex, sidx + " " + sord);

             // fields and titles
             string[,] fieldNames = new string[,] {
                 {"CategoryID", "Category ID"},
                 {"CategoryName", "Category Name"},
                 {"Description", "Description"}
             };

             List<string> unsortableFields = new List<string>();
             unsortableFields.Add("Description");

             // view model
             CategoriesForeachViewModel viewModel = new CategoriesForeachViewModel();
             viewModel.CategoriesData = objCategoriesCol;
             viewModel.CategoriesFieldNames = fieldNames;
             viewModel.TotalPages = totalPages;
             viewModel.CurrentPage = currentPage;
             viewModel.FieldToSort = String.IsNullOrEmpty(sidx) ? "CategoryID" : sidx;
             viewModel.FieldSortOrder = String.IsNullOrEmpty(sord) ? "asc" : sord;
             viewModel.FieldToSortWithOrder = String.IsNullOrEmpty(sidx) ? "CategoryID" : (sidx + " " + sord).Trim();
             viewModel.NumberOfPagesToShow = numberOfPagesToShow;
             viewModel.StartPage = Functions.GetPagerStartPage(currentPage, numberOfPagesToShow, totalPages);
             viewModel.EndPage = Functions.GetPagerEndPage(viewModel.StartPage, currentPage, numberOfPagesToShow, totalPages);
             viewModel.UnsortableFields = unsortableFields;

             return View(viewModel);
         }

         /// <summary>
         /// GET: /Categories/Unbound
         /// </summary>
         public IActionResult Unbound()
         {
             return View(GetUnboundViewModel());
         }

         /// <summary>
         /// POST: /Categories/Unbound
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public IActionResult Unbound(CategoriesViewModel viewModel, string returnUrl)
         {
             if (ModelState.IsValid)
             {
                 // do something here before redirecting

                 if (Url.IsLocalUrl(returnUrl))
                     return Redirect(returnUrl);
                 else
                     return RedirectToAction("Index", "Home");
             }

             // if we got this far, something failed, redisplay form
             return View(GetUnboundViewModel());
         }

         private CategoriesViewModel GetUnboundViewModel()
         {
             CategoriesViewModel viewModel = new CategoriesViewModel();
             viewModel.CategoriesModel = null;
             viewModel.ViewControllerName = "Categories";
             viewModel.ViewActionName = "Unbound";
             viewModel.ViewReturnUrl = "/Home";

             return viewModel;
         }

         /// <summary>
         /// GET: /Categories/AddEditCategories
         /// </summary>
         private void AddEditCategories(CategoriesViewModel viewModel, CrudOperation operation, bool isForListInline = false)
         {
             CategoriesModel model = viewModel.CategoriesModel;
             Categories objCategories;

             if (operation == CrudOperation.Add)
                objCategories = new Categories();
             else
                objCategories = Categories.SelectByPrimaryKey(model.CategoryID);

             objCategories.CategoryID = model.CategoryID;
             objCategories.CategoryName = model.CategoryName;
             objCategories.Description = model.Description;

             if (operation == CrudOperation.Add)
                objCategories.Insert();
             else
                objCategories.Update();
         }

         private CategoriesViewModel GetViewModel(string actionName)
         {
             CategoriesViewModel viewModel = new CategoriesViewModel();
             viewModel.CategoriesModel = null;
             viewModel.ViewControllerName = "Categories";
             viewModel.ViewActionName = actionName;

             return viewModel;
         }

         private List<Categories> GetFilteredData(int? categoryID, string categoryName, string sidx, string sord, int rows, int startRowIndex, string sortExpression)
         {
             if (categoryID != null || !String.IsNullOrEmpty(categoryName))
                 return Categories.SelectSkipAndTakeDynamicWhere(categoryID, categoryName, rows, startRowIndex, sortExpression);

             return Categories.SelectSkipAndTake(rows, startRowIndex, sortExpression);
         }

         /// <summary>
         /// GET: /Categories/GridData
         /// </summary>
         public IActionResult GridData(string sidx, string sord, int page, int rows, bool isforJqGrid = true)
         {
             int totalRecords = Categories.GetRecordCount();
             int startRowIndex = ((page * rows) - rows) + 1;
             List<Categories> objCategoriesCol = Categories.SelectSkipAndTake(rows, startRowIndex, sidx + " " + sord);
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             if (objCategoriesCol is null)
                 return Json("{ total = 0, page = 0, records = 0, rows = null }");

             var jsonData = new
             {
                 total = totalPages,
                 page,
                 records = totalRecords,
                 rows = (
                     from objCategories in objCategoriesCol
                     select new
                     {
                         id = objCategories.CategoryID,
                         cell = new string[] { 
                             objCategories.CategoryID.ToString(),
                             objCategories.CategoryName,
                             objCategories.Description
                         }
                     }).ToArray()
             };

             return Json(jsonData);
         }

         /// <summary>
         /// GET: /Categories/GridDataWithFilters
         /// </summary>
         public IActionResult GridDataWithFilters(string sidx, string sord, int page, int rows, string filters)
         {
             int? categoryID = null;
             string categoryName = String.Empty;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize json and get values being searched
                 var jsonResult = JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "categoryid")
                         categoryID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "categoryname")
                         categoryName = rule["data"].Value;

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (categoryID == -1)
                     categoryID = null;

             }

             int totalRecords = Categories.GetRecordCountDynamicWhere(categoryID, categoryName);
             int startRowIndex = ((page * rows) - rows) + 1;
             List<Categories> objCategoriesCol = Categories.SelectSkipAndTakeDynamicWhere(categoryID, categoryName, rows, startRowIndex, sidx + " " + sord);
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             if (objCategoriesCol is null)
                 return Json("{ total = 0, page = 0, records = 0, rows = null }");

             var jsonData = new
             {
                 total = totalPages,
                 page,
                 records = totalRecords,
                 rows = (
                     from objCategories in objCategoriesCol
                     select new
                     {
                         id = objCategories.CategoryID,
                         cell = new string[] { 
                             objCategories.CategoryID.ToString(),
                             objCategories.CategoryName,
                             objCategories.Description
                         }
                     }).ToArray()
             };

             return Json(jsonData);
         }
     }
}
