using Microsoft.AspNetCore.Mvc;
using System;
using System.Linq;
using NwndSpAPI.BusinessObject;
using NwndSpAPI.Models;
using NwndSpAPI.ViewModels;
using NwndSpAPI.Domain;
using System.Collections.Generic;
using Microsoft.Extensions.Options;
using System.Threading.Tasks;
using Newtonsoft.Json;
using System.Text;

namespace NwndSp.Controllers.Base
{
     /// <summary>
     /// Base class for SuppliersController.  Do not make changes to this class,
     /// instead, put additional code in the SuppliersController class 
     /// </summary>
     public class SuppliersControllerBase : Controller
     {

         /// <summary>
         /// GET: /Suppliers/
         /// </summary>
         public IActionResult Index()
         {
             return View();
         }

         /// <summary>
         /// GET: /Suppliers/Add
         /// </summary>
         public IActionResult Add()
         {
             return GetAddViewModel();
         }

         /// <summary>
         /// POST: /Suppliers/Add
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public IActionResult Add(SuppliersViewModel viewModel, string returnUrl)
         {
             if (ModelState.IsValid)
             {
                 try
                 {
                     // add new record
                     AddEditSuppliers(viewModel, CrudOperation.Add);

                     if (Url.IsLocalUrl(returnUrl))
                         return Redirect(returnUrl);
                     else
                         return RedirectToAction("ListCrudRedirect", "Suppliers");
                 }
                 catch(Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             return GetAddViewModel();
         }

         private IActionResult GetAddViewModel()
         {
             SuppliersViewModel viewModel = new SuppliersViewModel();
             viewModel.SuppliersModel = null;
             viewModel.Operation = CrudOperation.Add;
             viewModel.ViewControllerName = "Suppliers";
             viewModel.ViewActionName = "Add";
             viewModel.ViewReturnUrl = "/Suppliers/ListCrudRedirect";

             return View(viewModel);
         }

         /// <summary>
         /// GET: /Suppliers/Update/5
         /// </summary>
         public IActionResult Update(int id)
         {
             return GetUpdateViewModel(id);
         }

         /// <summary>
         /// POST: /Suppliers/Update/5
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public IActionResult Update(int id, SuppliersViewModel viewModel, string returnUrl)
         {
             if (ModelState.IsValid)
             {
                 try
                 {
                     // update record
                     AddEditSuppliers(viewModel, CrudOperation.Update);

                     if (Url.IsLocalUrl(returnUrl))
                         return Redirect(returnUrl);
                     else
                         return RedirectToAction("ListCrudRedirect", "Suppliers");
                 }
                 catch(Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             return GetUpdateViewModel(id);
         }

         public IActionResult GetUpdateViewModel(int id)
         {
             // select a record by primary key(s)
             Suppliers objSuppliers = Suppliers.SelectByPrimaryKey(id);

             // assign values to the model
             SuppliersModel model = new SuppliersModel();
             model.SupplierID = objSuppliers.SupplierID;
             model.CompanyName = objSuppliers.CompanyName;
             model.ContactName = objSuppliers.ContactName;
             model.ContactTitle = objSuppliers.ContactTitle;
             model.Address = objSuppliers.Address;
             model.City = objSuppliers.City;
             model.Region1 = objSuppliers.Region1;
             model.PostalCode = objSuppliers.PostalCode;
             model.Country = objSuppliers.Country;
             model.Phone = objSuppliers.Phone;
             model.Fax = objSuppliers.Fax;
             model.HomePage = objSuppliers.HomePage;

             // assign values to the view model
             SuppliersViewModel viewModel = new SuppliersViewModel();
             viewModel.SuppliersModel = model;
             viewModel.Operation = CrudOperation.Update;
             viewModel.ViewControllerName = "Suppliers";
             viewModel.ViewActionName = "Update";

             viewModel.ViewReturnUrl = "/Suppliers/ListCrudRedirect";
             viewModel.URLReferrer = Request.Headers["Referer"].ToString();

             return View(viewModel);
         }

         /// <summary>
         /// POST: /Suppliers/Delete/5
         /// </summary>
         [HttpPost]
         public IActionResult Delete(int id, SuppliersViewModel viewModel, string returnUrl)
         {
             Suppliers.Delete(id);
             return Json(true);
         }

         /// <summary>
         /// GET: /Suppliers/Details/5
         /// </summary>
         public IActionResult Details(int id)
         {
             // select a record by primary key(s)
             Suppliers objSuppliers = Suppliers.SelectByPrimaryKey(id);

             // assign values to the model
             SuppliersModel model = new SuppliersModel();
             model.SupplierID = objSuppliers.SupplierID;
             model.CompanyName = objSuppliers.CompanyName;
             model.ContactName = objSuppliers.ContactName;
             model.ContactTitle = objSuppliers.ContactTitle;
             model.Address = objSuppliers.Address;
             model.City = objSuppliers.City;
             model.Region1 = objSuppliers.Region1;
             model.PostalCode = objSuppliers.PostalCode;
             model.Country = objSuppliers.Country;
             model.Phone = objSuppliers.Phone;
             model.Fax = objSuppliers.Fax;
             model.HomePage = objSuppliers.HomePage;

             // assign values to the view model
             SuppliersViewModel viewModel = new SuppliersViewModel();
             viewModel.SuppliersModel = model;

             viewModel.ViewReturnUrl = "/Suppliers/ListCrudRedirect";
             viewModel.URLReferrer = Request.Headers["Referer"].ToString();

             return View(viewModel);
         }

         /// <summary>
         /// GET: /Suppliers/ListCrudRedirect
         /// </summary>
         public IActionResult ListCrudRedirect()
         {
             return View();
         }

         /// <summary>
         /// GET: /Suppliers/ListReadOnly
         /// </summary>
         public IActionResult ListReadOnly()
         {
             return View();
         }

         /// <summary>
         /// GET: /Suppliers/ListCrud
         /// </summary>
         public IActionResult ListCrud()
         {
             return View(GetViewModel("ListCrud"));
         }

         /// <summary>
         /// POST: /Suppliers/ListCrud
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public IActionResult ListCrud(string inputSubmit, SuppliersViewModel viewModel)
         {
             if (ModelState.IsValid)
             {
                 CrudOperation operation = CrudOperation.Add;

                 if (inputSubmit == "Update")
                     operation = CrudOperation.Update;

                 try
                 {
                     AddEditSuppliers(viewModel, operation);
                 }
                 catch(Exception ex)
                 {
                     ModelState.AddModelError("", ex.Message);
                 }
             }

             return View(GetViewModel("ListCrud"));
         }

         /// <summary>
         /// GET: /Suppliers/ListSearch
         /// </summary>
         public IActionResult ListSearch()
         {
             return View(GetViewModel("ListSearch"));
         }

         /// <summary>
         /// GET: /Suppliers/ListScrollLoad
         /// </summary>
         public IActionResult ListScrollLoad()
         {
             return View();
         }

         /// <summary>
         /// GET: /Suppliers/ListInline
         /// </summary>
         public IActionResult ListInline()
         {
             return View(GetViewModel("ListInline"));
         }

         /// <summary>
         /// POST: /Suppliers/ListInlineAdd
         /// </summary>
         [HttpPost]
         public IActionResult ListInlineAdd([FromBody]SuppliersModel model)
         {
             SuppliersViewModel viewModel = new SuppliersViewModel();
             viewModel.SuppliersModel = model;

             AddEditSuppliers(viewModel, CrudOperation.Add, true);
             return Json("");
         }

         /// <summary>
         /// POST: /Suppliers/ListInlineUpdate
         /// </summary>
         [HttpPost]
         public IActionResult ListInlineUpdate([FromBody]SuppliersModel model)
         {
             SuppliersViewModel viewModel = new SuppliersViewModel();
             viewModel.SuppliersModel = model;

             AddEditSuppliers(viewModel, CrudOperation.Update, true);
             return Json("");
         }

         /// <summary>
         /// GET: /Suppliers/ListForeach
         /// </summary>
         public IActionResult ListForeach(string sidx, string sord, int? page)
         {
             int rows = Functions.GetGridNumberOfRows();
             int numberOfPagesToShow = Functions.GetGridNumberOfPagesToShow();
             int currentPage = page is null ? 1 : Convert.ToInt32(page);
             int startRowIndex = ((currentPage * rows) - rows) + 1;
             int totalRecords = Suppliers.GetRecordCount();
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);
             List<Suppliers> objSuppliersCol = Suppliers.SelectSkipAndTake(rows, startRowIndex, sidx + " " + sord);

             // fields and titles
             string[,] fieldNames = new string[,] {
                 {"SupplierID", "Supplier ID"},
                 {"CompanyName", "Company Name"},
                 {"ContactName", "Contact Name"},
                 {"ContactTitle", "Contact Title"},
                 {"Address", "Address"},
                 {"City", "City"},
                 {"Region1", "Region1"},
                 {"PostalCode", "Postal Code"},
                 {"Country", "Country"},
                 {"Phone", "Phone"},
                 {"Fax", "Fax"},
                 {"HomePage", "Home Page"}
             };

             List<string> unsortableFields = new List<string>();
             unsortableFields.Add("HomePage");

             // view model
             SuppliersForeachViewModel viewModel = new SuppliersForeachViewModel();
             viewModel.SuppliersData = objSuppliersCol;
             viewModel.SuppliersFieldNames = fieldNames;
             viewModel.TotalPages = totalPages;
             viewModel.CurrentPage = currentPage;
             viewModel.FieldToSort = String.IsNullOrEmpty(sidx) ? "SupplierID" : sidx;
             viewModel.FieldSortOrder = String.IsNullOrEmpty(sord) ? "asc" : sord;
             viewModel.FieldToSortWithOrder = String.IsNullOrEmpty(sidx) ? "SupplierID" : (sidx + " " + sord).Trim();
             viewModel.NumberOfPagesToShow = numberOfPagesToShow;
             viewModel.StartPage = Functions.GetPagerStartPage(currentPage, numberOfPagesToShow, totalPages);
             viewModel.EndPage = Functions.GetPagerEndPage(viewModel.StartPage, currentPage, numberOfPagesToShow, totalPages);
             viewModel.UnsortableFields = unsortableFields;

             return View(viewModel);
         }

         /// <summary>
         /// GET: /Suppliers/Unbound
         /// </summary>
         public IActionResult Unbound()
         {
             return View(GetUnboundViewModel());
         }

         /// <summary>
         /// POST: /Suppliers/Unbound
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public IActionResult Unbound(SuppliersViewModel viewModel, string returnUrl)
         {
             if (ModelState.IsValid)
             {
                 // do something here before redirecting

                 if (Url.IsLocalUrl(returnUrl))
                     return Redirect(returnUrl);
                 else
                     return RedirectToAction("Index", "Home");
             }

             // if we got this far, something failed, redisplay form
             return View(GetUnboundViewModel());
         }

         private SuppliersViewModel GetUnboundViewModel()
         {
             SuppliersViewModel viewModel = new SuppliersViewModel();
             viewModel.SuppliersModel = null;
             viewModel.ViewControllerName = "Suppliers";
             viewModel.ViewActionName = "Unbound";
             viewModel.ViewReturnUrl = "/Home";

             return viewModel;
         }

         /// <summary>
         /// GET: /Suppliers/AddEditSuppliers
         /// </summary>
         private void AddEditSuppliers(SuppliersViewModel viewModel, CrudOperation operation, bool isForListInline = false)
         {
             SuppliersModel model = viewModel.SuppliersModel;
             Suppliers objSuppliers;

             if (operation == CrudOperation.Add)
                objSuppliers = new Suppliers();
             else
                objSuppliers = Suppliers.SelectByPrimaryKey(model.SupplierID);

             objSuppliers.SupplierID = model.SupplierID;
             objSuppliers.CompanyName = model.CompanyName;
             objSuppliers.ContactName = model.ContactName;
             objSuppliers.ContactTitle = model.ContactTitle;
             objSuppliers.Address = model.Address;
             objSuppliers.City = model.City;
             objSuppliers.Region1 = model.Region1;
             objSuppliers.PostalCode = model.PostalCode;
             objSuppliers.Country = model.Country;
             objSuppliers.Phone = model.Phone;
             objSuppliers.Fax = model.Fax;
             objSuppliers.HomePage = model.HomePage;

             if (operation == CrudOperation.Add)
                objSuppliers.Insert();
             else
                objSuppliers.Update();
         }

         private SuppliersViewModel GetViewModel(string actionName)
         {
             SuppliersViewModel viewModel = new SuppliersViewModel();
             viewModel.SuppliersModel = null;
             viewModel.ViewControllerName = "Suppliers";
             viewModel.ViewActionName = actionName;

             return viewModel;
         }

         private List<Suppliers> GetFilteredData(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, string sidx, string sord, int rows, int startRowIndex, string sortExpression)
         {
             if (supplierID != null || !String.IsNullOrEmpty(companyName) || !String.IsNullOrEmpty(contactName) || !String.IsNullOrEmpty(contactTitle) || !String.IsNullOrEmpty(address) || !String.IsNullOrEmpty(city) || !String.IsNullOrEmpty(region1) || !String.IsNullOrEmpty(postalCode) || !String.IsNullOrEmpty(country) || !String.IsNullOrEmpty(phone) || !String.IsNullOrEmpty(fax))
                 return Suppliers.SelectSkipAndTakeDynamicWhere(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, rows, startRowIndex, sortExpression);

             return Suppliers.SelectSkipAndTake(rows, startRowIndex, sortExpression);
         }

         /// <summary>
         /// GET: /Suppliers/GridData
         /// </summary>
         public IActionResult GridData(string sidx, string sord, int page, int rows, bool isforJqGrid = true)
         {
             int totalRecords = Suppliers.GetRecordCount();
             int startRowIndex = ((page * rows) - rows) + 1;
             List<Suppliers> objSuppliersCol = Suppliers.SelectSkipAndTake(rows, startRowIndex, sidx + " " + sord);
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             if (objSuppliersCol is null)
                 return Json("{ total = 0, page = 0, records = 0, rows = null }");

             var jsonData = new
             {
                 total = totalPages,
                 page,
                 records = totalRecords,
                 rows = (
                     from objSuppliers in objSuppliersCol
                     select new
                     {
                         id = objSuppliers.SupplierID,
                         cell = new string[] { 
                             objSuppliers.SupplierID.ToString(),
                             objSuppliers.CompanyName,
                             objSuppliers.ContactName,
                             objSuppliers.ContactTitle,
                             objSuppliers.Address,
                             objSuppliers.City,
                             objSuppliers.Region1,
                             objSuppliers.PostalCode,
                             objSuppliers.Country,
                             objSuppliers.Phone,
                             objSuppliers.Fax,
                             objSuppliers.HomePage
                         }
                     }).ToArray()
             };

             return Json(jsonData);
         }

         /// <summary>
         /// GET: /Suppliers/GridDataWithFilters
         /// </summary>
         public IActionResult GridDataWithFilters(string sidx, string sord, int page, int rows, string filters)
         {
             int? supplierID = null;
             string companyName = String.Empty;
             string contactName = String.Empty;
             string contactTitle = String.Empty;
             string address = String.Empty;
             string city = String.Empty;
             string region1 = String.Empty;
             string postalCode = String.Empty;
             string country = String.Empty;
             string phone = String.Empty;
             string fax = String.Empty;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize json and get values being searched
                 var jsonResult = JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "supplierid")
                         supplierID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "companyname")
                         companyName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "contactname")
                         contactName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "contacttitle")
                         contactTitle = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "address")
                         address = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "city")
                         city = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "region1")
                         region1 = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "postalcode")
                         postalCode = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "country")
                         country = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "phone")
                         phone = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "fax")
                         fax = rule["data"].Value;

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (supplierID == -1)
                     supplierID = null;

             }

             int totalRecords = Suppliers.GetRecordCountDynamicWhere(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
             int startRowIndex = ((page * rows) - rows) + 1;
             List<Suppliers> objSuppliersCol = Suppliers.SelectSkipAndTakeDynamicWhere(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, rows, startRowIndex, sidx + " " + sord);
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             if (objSuppliersCol is null)
                 return Json("{ total = 0, page = 0, records = 0, rows = null }");

             var jsonData = new
             {
                 total = totalPages,
                 page,
                 records = totalRecords,
                 rows = (
                     from objSuppliers in objSuppliersCol
                     select new
                     {
                         id = objSuppliers.SupplierID,
                         cell = new string[] { 
                             objSuppliers.SupplierID.ToString(),
                             objSuppliers.CompanyName,
                             objSuppliers.ContactName,
                             objSuppliers.ContactTitle,
                             objSuppliers.Address,
                             objSuppliers.City,
                             objSuppliers.Region1,
                             objSuppliers.PostalCode,
                             objSuppliers.Country,
                             objSuppliers.Phone,
                             objSuppliers.Fax,
                             objSuppliers.HomePage
                         }
                     }).ToArray()
             };

             return Json(jsonData);
         }
     }
}
